@@ -23,9 +23,7 @@
 
 #include "pavucontrol.h"
 
-#include <canberra-gtk.h>
-
-class StreamWidget;
+class MinimalStreamWidget;
 
 class ChannelWidget : public Gtk::EventBox {
 public:
@@ -39,15 +37,13 @@ public:
     Gtk::HScale *volumeScale;
 
     int channel;
-    StreamWidget *streamWidget;
+    MinimalStreamWidget *minimalStreamWidget;
 
     void onVolumeScaleValueChanged();
 
     bool can_decibel;
     bool volumeScaleEnabled;
 
-    Glib::ustring beepDevice;
-
     virtual void set_sensitive(bool enabled);
     virtual void setBaseVolume(pa_volume_t);
     virtual void setSteps(unsigned n);
